# main.py
# -*- coding: utf-8 -*-

import os.path
from qt import *
from qtcanvas import *
from mapview import MapView
from filedialog import FileDialog
from infodialog import InfoDialog
from pointsdialog import PointsDialog

_caption = 'ZMetro Editor'

class mainForm(QMainWindow):
    def __init__(self,pics_path,parent = None,name = None,fl = 0):
        QMainWindow.__init__(self,parent,name,fl)
        self.setCaption(_caption)
        self.toolBar = QToolBar(self)
        self.toolBar.setHorizontalStretchable(True)
##         setToolBarMovable(False)
        self.searchList = QComboBox(self.toolBar)
        self.searchList.setInsertionPolicy(QComboBox.NoInsertion)
        self.searchList.setSizePolicy(QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred, False))
        self.toolBar.setStretchableWidget(self.searchList)

        self.connect(self.searchList,SIGNAL("activated(const QString &)"), self.findPoint)
        
        self.btnSelect = QToolButton(QPixmap(QString(os.path.join(pics_path, "flag.png"))), "Edit", None, self.selectPoint, self.toolBar)
        self.btnSelect.setToggleButton(True)
        self.toolBar.addSeparator()
        self.btnOpen = QToolButton(QPixmap(QString(os.path.join(pics_path, "fileopen.png"))), "Open", None, self.openMap, self.toolBar)
        self.btnSave = QToolButton(QPixmap(QString(os.path.join(pics_path, "DSave.png"))), "Save", None, self.saveMap, self.toolBar)
        self.btnInfo = QToolButton(QPixmap(QString(os.path.join(pics_path, "SettingsIcon.png"))), "Info", None, self.editInfo, self.toolBar)
        self.enableButtons(False)
        self.layout = QVBoxLayout(self)
        self.can = QCanvas(self)
        self.view = MapView(self.tapped, self.can, self)
        self.view.setCanvas(self.can)
        self.setCentralWidget(self.view)
        self.layout.addWidget(self.view)

    def enableButtons(self, val):
        self.btnSave.setEnabled(val)
        self.btnSelect.setEnabled(val)
        self.btnInfo.setEnabled(val)
        self.searchList.setEnabled(val)

    def openMap(self):
        dlg = FileDialog(self, 'OpenFile', 'Open File', filter='*.png|*.jpg|*.gif')
        if dlg.exec_loop() and dlg.selectedFile():
            self.openFile(dlg.selectedFile())
            self.enableButtons(True)

    def openFile(self, fname):
        import os.path
        (name, ext) = os.path.splitext(fname)
        (path, iname) = os.path.split(fname)
        mname = os.path.join(path, 'description.map')
        self.loadDescription(mname)
        self.info['image'] = iname
        p = QPixmap(QString(fname))
        self.can.resize(p.width(), p.height())
        self.can.setBackgroundPixmap(p)
        self.setCaption(QString().fromUtf8(_caption + ' : ' + self.info['name']))

    def saveMap(self):
        points = {}
        for (name, xy) in self.points.items():
            val = '%d,%d' % xy
            points[name] = val
        f = open(self.mapName, 'w')
        self.writeSection(f, 'info', self.info)
        self.writeSection(f, 'points', points)
        f.close()

    def writeSection(self, f, section, data):
        f.write('[%s]\n' % section)
        keys = data.keys()
        keys.sort()
        for key in keys:
            f.write('%s=%s\n' % (key, data[key]))
        f.write('\n')

    def loadDescription(self, fname):
        import os.path
        from ConfigParser import ConfigParser
        self.mapName = fname
        self.info = {'name': '', 'description': '', 'copyright': '',
                     'image': ''}
        self.points = {}
        if not os.path.exists(fname):
            print 'Cannot find map file', fname
            return

        cfg = ConfigParser()
        cfg.read(fname)
        if cfg.has_section('info'):
            for (name, val) in cfg.items('info'):
##                 print '>>>>', name, ':', val
                self.info[name] = val
        self.searchList.clear()
        if cfg.has_section('points'):
            for (name, val) in cfg.items('points'):
                try:
                    (sx, sy) = val.split(',')
                    x = int(sx.strip())
                    y = int(sy.strip())
                except:
                    raise IOError, 'Format must be: name=x,y where x, y must be integers!'
                self.points[name] = (x, y)
##                 print '>>>>', name, x, y
            self.fillSearchList()

    def fillSearchList(self):
        self.searchList.clear()
        for item in self.getPointList():
            self.searchList.insertItem(QString().fromUtf8(item))
        self.searchList.setCurrentItem(-1)

    def getPointList(self):
        points = self.points.keys()
        points.sort()
        return points

    def tapped(self, e):
        self.points[self.selectedPoint] = (e.x(), e.y())
        self.fillSearchList()
        self.searchList.setEnabled(True)
        self.btnSelect.setOn(False)

    def selectPoint(self):
        self.btnSelect.setOn(self.view.drag)
        if not self.view.drag:
            self.fillSearchList()
            self.searchList.setEnabled(True)
            self.view.dragOn()
        else:
            dlg = PointsDialog(self.getPointList(), True, self)
            if dlg.exec_loop():
                self.selectedPoint = dlg.selected()
                self.searchList.insertItem(QString().fromUtf8(self.selectedPoint))
                self.searchList.setCurrentItem(self.searchList.count()-1)
                self.searchList.setEnabled(False)
                self.view.dragOff()
            else:
                self.fillSearchList()
                self.searchList.setEnabled(True)
                self.view.dragOn()
                self.btnSelect.setOn(False)

    def findPoint(self, txt):
        point = str(txt.utf8())
        (x, y) = self.points[point]
        self.view.center(x, y)
##         print '****', point, x, y

    def editInfo(self):
        dlg = InfoDialog(self)
        dlg.addEdit('name', 'Name:', self.info['name'])
        dlg.addEdit('copyright', 'Copyright:', self.info['copyright'])
        dlg.addEdit('description', 'Description:', self.info['description'])
        if dlg.exec_loop():
            for (key, edit) in dlg.editBoxes.items():
                value = str(edit.text().utf8())
                self.info[key] = value
            self.setCaption(QString().fromUtf8(_caption + ' : ' + self.info['name']))
##                 print '****', key, ':', value

# end of main.py
